% Program name: basic_example1.m;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Example GMM Estimation                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following program uses the firm's euler equation to estimate the ajustment cost parameter (gamma) 
% and the capital share (alpha). See Adda Cooper (2005) section 8.4.3 for a
% discussion of  this model (page 199 and following).

% with quadratic adjustment costs the euler equation for investment is
% i(t) = 1/ gamma (\beta ( E(t) (alpha*(A(t+1)K(t+1)^alpha-1)) + p(t+1)(1-delta) +
% 0.5*gamma* i(t+1)^2+ gamma(1-delta) i(t+1))- p(t))
% the parameters of interest are gamma and alpha (beta and delta will be
% calibrated). Note in the routine we normalize the price p(t)=p=1 for all t.

% NOTE: the program has two options for the minimization routine: 
% fminsearch which does not produce standard error of the estimates 
% fminunc which produces standard errors but may not work whe the objective function
% is very flat.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% exercise 1: fix alpha to 0.5 and find optimal gamma
% exercise 2: jointly estimate alpha, gamma, delta
% exercise 3: use more instruments (instruments are defined in the
%              file model1)
% exercise 4: check if results are different with a two-step estimator.
% exercise 5: allow for serial correlation in the error (need to change
%              the definition of W in varcov) 
% exercise 6: cut sample size by half (use the last 190 data points)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
clear all;                          % Clears all variables
clc                                 % Clears the command window

% Start the timer
tic
% Load the data set
% invpc = investment rate (i)  
% shc = productivity shock (A)
% prok = profit rate (AK^alpha-1)
load book2;
data = [invpc; shc; prok];

% options: Choose the routine to be used in the minimization
routine = 1; 

% Parameters calibrate betta delta, estimate alpha gamma
betta = .95;   
delta = .069;
guess = [1 1]; % this is the initial guess for alpha, gamma
p = 1;                      % Normalize price to one
T = length(data);
maxiter = 10;           % Convergence is fast therefore only need two iterations
                        % this is a parameter to play with
iter = 1;
W = eye(2);             % Step One of the GMM algorithm

if routine == 1
    while iter <= maxiter;
        [results,J,ef,out,grad,Hessian]= fminunc(@model1,guess,[],data,betta,delta,p,T,W);
        gamma = results(1);
        alpha = results(2);
        stderrors = sqrt(diag(Hessian));
    
        % Calculate the Optimal Weight Matrix (Step Two)
        k=2;   % order of autocovariances to include in optimal W
        W = varcovx(gamma,alpha,data,betta,delta,p,T,k);
        iter = iter +1;
    end
disp('*****************')
disp('     Results')
disp('*****************')
disp(['Gamma = ' num2str(gamma) ' (' num2str(stderrors(1)) ')'])
disp(['Alpha = ' num2str(alpha) ' (' num2str(stderrors(2)) ')'])
disp(['J statistic = ' num2str(T*J)])
disp('*****************')
else
    while iter <= maxiter;
        [results J]= fminsearch(@model1,guess,[],data,betta,delta,p,T,W);
        gamma = results(1);
        alpha = results(2);
    
        % Calculate the Optimal Weight Matrix (Step Two)
        W = varcov(gamma,alpha,data,betta,delta,p,T);
        iter = iter +1;
    end
disp('*****************')
disp('     Results')
disp('*****************')
disp(['Gamma = ' num2str(gamma)])
disp(['Alpha = ' num2str(alpha)])
disp(['J statistic = ' num2str(T*J)])
disp('*****************')
end

% End the timer
toc